/*************************************************************************/
/*************************************************************************/
/*                                                                       */
/*  File Name : gene.cc                                                  */
/*                                                                       */
/*       Date : November  6, 1997                                        */
/* Updated at : November 24, 1997                                        */
/*                                                                       */
/*************************************************************************/
/*************************************************************************/

#include "gene.h"
#include <string.h>
#include <math.h>


/*
// Defines a Gene and returns its memory address.
*/

type_Gene* defineGene(char *_name, coord_t _x, coord_t _y,
                        unsigned int _index )
{
   type_Gene * mygene = (type_Gene *) calloc(1,sizeof(type_Gene));

   if(mygene!=NULL)
   {

      char* myname = (char *) calloc(nameLength,sizeof(char));

      if(myname!=NULL)
      {
         strncpy(myname, _name, nameLength);
         mygene->name = myname;
         mygene->x = _x;
         mygene->y = _y;
         mygene->index = _index;
      }
      else 
      {
         free(mygene);
         mygene = NULL;
      }

   }// end of if

   return mygene; 

}// end of 

/*
// Calculates Eucledian distance between this gene and
// _gene and returns the value.
*/

distance_t calculateDistanceTo(type_Gene* from, type_Gene* to)
{
   distance_t dist = 
              sqrt(pow((from->x - to->x), 2) + pow((from->y - to->y), 2));
   
   return dist;

}// end of calculateDistanceTo()

/*
// Returns Eucledian distance between this gene and _gene.
*/

distance_t distanceTo(type_Gene* from, type_Gene* to)
{
   
   return calculateDistanceTo(from,to);

}// end of distanceTo()

/*
// Compares two Genes
*/

int
isEqual(type_Gene a, type_Gene b)
{

    int i, ret = 0;

    ret = ( a.x == b.x ) && ( a.y == b.y ) && ( a.index == b.index );

    for(i=0;ret && i<nameLength;i++) ret = ( a.name[i] == b.name[i] );
     
    return ret;
    
}// end of isEqual()

int printGene(type_Gene a)
{

   printf(" %3d %f %f",a.index,a.x,a.y);

}//

int getIndex(type_Gene this) { return this.index; };

char *getName(type_Gene this) { return this.name; } ;

void disposeGene(type_Gene *this)
{
   free(this->name);
   free(this);
}

